/*
plugin name: Armin Attribute Holder
Class: Modifier

Author : Armin Soleiman Boroujerdi
E-mail: Armin.soleimani.87@gmail.com
Website: www.ArminSoleimani.net

written on: 3Ds Max 2015
starting date: June 2017
------------------------------

Definition:
This plugin is desined to "handle" different kind of "values" in 3Ds Max. "Handling" means doing actions like Save ,Restore and merge parameters,
And "Values" means position, Rotation and Custom Attributes as long as they are a valid controller. 
This Plugin can handle this type of controllers:
-Position List
-Position XYZ
-Rotation List
-Rotation XYZ
-Custom Attributes
***Note: it only supports the fist sub-level of a list controller :
			for example :position > position list > position XYZ (supported)
			any further sub lists, are treaded by its first parent under the main list controller:
			for example :position > position list > position List-subcontroller > position XYZ (supported only at level of "position List-subcontroller")
			
			also for float controllers, in only sees a "float list" and its sub-lists as a single value, and doesn't act on every single sublist individually.
			
			
			
there is no limitation in number of Controllers/attributes and the number of saved presets. 


*/
plugin modifier 'Armin Attribute Holder'
	name:"Armin Attribute Holder"
	classID:#(0x1c31dad, 0x784ffe5a)
	version:2
(
	local ParamColl=#() , thisObject =undefined ,Double_ctrl_check ,indexStart, indexEnd
	local initialPosVals=#() ,finalPosVals=#() ,finalRotVals=#(), initialRotVals=#(),finalAttVals=#(), initialAttVals=#()
	local initialSelfAttVals = #(),	finalSelfAttVals = #()
	parameters mod_param
	(
		Position_param	type:#MaxObjectTab	tabsizevariable:true	tabSize:0 animatable:true
		Position_param_vals type:#point3tab tabsizevariable:true	tabSize:0 animatable:true
		Pos_Indexnum	type:#inttab tabsizevariable:true	tabSize:0
		
		rotation_param	type:#MaxObjectTab	tabsizevariable:true	tabSize:0 animatable:true
		rotation_param_vals  type:#matrix3tab tabsizevariable:true	tabSize:0 animatable:true
		Rot_Indexnum	type:#inttab tabsizevariable:true	tabSize:0
		
		float_Param 		type:#MaxObjectTab	tabsizevariable:true	tabSize:0 animatable:true
		float_Param_val type:#floattab tabsizevariable:true	tabSize:0 animatable:true
		float_Indexnum	type:#inttab tabsizevariable:true	tabSize:0
		
		SelfAtt_Param 		type:#MaxObjectTab	tabsizevariable:true	tabSize:0 animatable:true
		SelfAtt_Param_val type:#floattab tabsizevariable:true	tabSize:0 animatable:true
		selfAtt_Indexnum	type:#inttab tabsizevariable:true	tabSize:0
		
		objects_col		type:#nodeTab	tabsizevariable:true
		CustAttobjects_col		type:#nodeTab	tabsizevariable:true 
	)
fn getIndexes valType =
(
	state_num = this.AttHolderRollout.ddl_states.selection
	case valType of
	(
		"pos":
		(
			indexEnd = 0
			for i= 1 to state_num do
			(
				indexEnd += Pos_Indexnum[i]
			)
			indexStart = 1
			if 
			state_num > 1 do
			(
				for i= 1 to state_num-1 do
					indexStart+= Pos_Indexnum[i]
				
			)
		)
		"rot":
		(
			indexEnd = 0
			for i= 1 to state_num do
			(
				indexEnd += Rot_Indexnum[i]
			)
			indexStart = 1
			if 
			state_num > 1 do
			(
				for i= 1 to state_num-1 do
					indexStart+= Rot_Indexnum[i]
				
			)
		)
		"CustParam":
		
		(
			indexEnd = 0
			for i= 1 to state_num do
			(
				indexEnd += float_Indexnum[i]
			)
			indexStart = 1
			if 
			state_num > 1 do
			(
				for i= 1 to state_num-1 do
					indexStart+= float_Indexnum[i]
				
			)
		)
		"SelfCustAtt":
		(
			
			indexEnd = 0
			for i= 1 to state_num do
			(
				indexEnd += selfAtt_Indexnum[i]
			)
			indexStart = 1
			if 
			state_num > 1 do
			(
				for i= 1 to state_num-1 do
					indexStart+= selfAtt_Indexnum[i]
				
			)
		)
	)
	
)
fn getObjects obj =
(
	testDub = 0
	for x in objects_col where x==obj while testDub==0 do
		testDub = 1
	
	if testDub == 0 do append objects_col obj
		
	print objects_col
)
fn ReturnObjects type=
(
	tempsel =#()
	case type of
	(
		"pos":
		if Position_param_vals.count !=0 do
		(
			getIndexes "pos"
			
			for i = 1 to (indexEnd-indexStart)+1 do
			(
				for j = 1 to objects_col.count do
				(
					if classof objects_col[j].position.controller == position_list do
					(
						a = objects_col[j].position.controller.count
						for q = 1 to a do
						(
							if objects_col[j].position.controller[q].controller == Position_param[i] do
							(
								append tempsel objects_col[j]
								
							)
						)
					)
					if classof objects_col[j].position.controller == Position_XYZ do
					(
						if objects_col[j].position.controller ==Position_param[i] do
							append tempsel objects_col[j]
					)
				)
			)
		)
		"rot": --print Rot_Indexnum
		
		if Rotation_param_vals.count !=0 do
		(
			getIndexes "rot"
			
			--print indexEnd
			--print indexStart
			for i = 1 to (indexEnd-indexStart)+1 do
			(
				print objects_col
				for j = 1 to objects_col.count do
				(
					if classof objects_col[j].rotation.controller == rotation_list do
					(
						a = objects_col[j].rotation.controller.count
						for q = 1 to a do
						(
							if objects_col[j].rotation.controller[q].controller == Rotation_param[i] do
							(
								append tempsel objects_col[j]
								
							)
						)
					)
					if classof objects_col[j].rotation.controller == Euler_XYZ do
					(
						if objects_col[j].rotation.controller ==Rotation_param[i] do
							append tempsel objects_col[j]
					)
				)
			)
			
		)
		
	)

	selectmore tempsel
	--if tempsel.count != 0 do deselect thisObject
	
)

fn KeyPreset type =
(
	case type of
	(
		"pos":
		if Position_param_vals.count !=0 do
		(
			GetIndexes "pos"
			for i = 1 to (indexEnd-indexStart)+1 do
			(
				Position_param[i].value = Position_param[i].value
			)
		)
		
		"rot":
		if rotation_param_vals.count !=0 do
		(
			GetIndexes "rot"
			for i = 1 to (indexEnd-indexStart)+1 do
			(
				rotation_param[i].value = rotation_param[i].value
			)
		)
		
		"CustParam":
		
		(
			if float_Param_val.count !=0 do
			(
				GetIndexes "CustParam"
				for i = 1 to (indexEnd-indexStart)+1 do
				(
					float_Param[i].value = float_Param[i].value
				)
			)
			if SelfAtt_Param_val.count !=0 do
			(
				GetIndexes "SelfCustAtt"
				for i = 1 to (indexEnd-indexStart)+1 do
				(
					SelfAtt_Param[i].value = SelfAtt_Param[i].value
				)
			)
		)
	)
)

fn get_Pos_Param =
(
	--get Position
	for i=1 to selection.count do
	(
		if classOf selection[i].position.controller == Position_list then
		(
			Pos_listStart = 1
			if this.AttHolderRollout.chk_getZeroPos.state == false do  Pos_listStart = 2
			for j = Pos_listStart to selection[i].position.controller.count do
			(
				--append Position_param selection[i].position.controller[j]
				test = Double_ctrl_check pos (selection[i].position.controller[j].controller)
			
				if test == true and classOf selection[i].position.controller[j].controller == Position_xyz do
				(
						append Position_param selection[i].position.controller[j].controller
						getObjects selection[i]
				)
				
			)
		)
		else
		(
			if classof selection[i].position.controller == Position_xyz do
			(
				test = Double_ctrl_check pos (selection[i].position.controller)
				if test == true do 
				(
					append Position_param selection[i].position.controller
					getObjects selection[i]
				)
					
			)
		)
		-- Test : 
		if classof selection[i].position.controller != Position_list and
			classof selection[i].position.controller != Position_xyz do
		(
			a= selection[i].name 
		messagebox ("Object : \" " + a +"\" , Position is not supported.") title:"Unsupported Position Controller"
			
		)
	)

)

fn get_Rotation_param =
(
	for i=1 to selection.count do
	(
		if classOf selection[i].rotation.controller == rotation_list then
		(
			Rot_listStart = 1
			if this.AttHolderRollout.chk_getZeroRot.state == false do  Rot_listStart = 2
				
			for j = Rot_listStart to selection[i].rotation.controller.count do
			(
				test = Double_ctrl_check rotation (selection[i].rotation.controller[j].controller)
				if test == true and classOf selection[i].rotation.controller[j].controller == Euler_XYZ do
				(
					append rotation_param selection[i].rotation.controller[j].controller
					getObjects selection[i]
				)
				
			)
		)
		else
		(
			if classof selection[i].rotation.controller == Euler_xyz do
			(
				test = Double_ctrl_check rotation (selection[i].rotation.controller)
				if test == true do
				(
					append rotation_param selection[i].rotation.controller
					getObjects selection[i]
				)
			)
			
		)
		-- Test : 
		if classof selection[i].rotation.controller != rotation_list and
			classof selection[i].rotation.controller != Euler_xyz do
				(
				a= selection[i].name 
				messagebox ("Object : \" " + a +"\" , Rotation is not supported.") title:"Unsupported Rotation Controller"
					
				)
	)
)
fn get_custAtt =
(
	--get for other objects
	for x = 1 to selection.count do
		--if selection[x]
	(
		if selection[x].modifiers.count >0 do
			for y = 1 to selection[x].modifiers.count do
			(
				if classof selection[x].modifiers[y] == EmptyModifier or
					classof selection[x].modifiers[y] == 'PEN_Attribute_Holder 2'do
				(
					for i= 1 to (custAttributes.count selection[x].modifiers[y]) do
						(
							
							a = custAttributes.get selection[x].modifiers[y] i
							TestUndefined = 0
							for j = 1 to a.Numsubs while TestUndefined == 0 do
							(
							
								--append float_Param a[j]
								
								--while TestUndefined == 0 do
								(
									if a[j].controller == undefined do 
									(
										
										TestUndefined = 1
									)
									
								)
								if testUndefined ==1 then
								(
									ss = stringstream ""
									format "Object: \n\"" to:ss
									format "%" selection[x].name to:ss
									format "\"\nModifier: \n\"" to:ss
									format "%" selection[x].modifiers[y].name to:ss
									format  "\" (Stack : " to:ss
									format (y as string) to:ss
									format ")\nHas Undefined Custom Attributes." to:ss
									
									messagebox ss title:"Undefined float"
								)
								else
								(
									test = Double_ctrl_check float a[j].controller
									if test == true do
									(
										append float_Param a[j].controller
										
										dubtest = 0
										for y in CustAttobjects_col while dubtest==0 where y==selection[x] do
											dubtest = 1
										
										if dubtest == 0 do append CustAttobjects_col selection[x]
									)
								)
							)
							
						)
				)
			)
	)
	
)
fn Get_currentAtt =
(
	SelfAtt_Param =#()
	
	selfAttCount = custAttributes.count this
	test = 0
	for i = 1 to selfAttCount while test==0 do
	(
		selfAtt = (custAttributes.get this i)
		for j =1 to  selfAtt.numsubs while test==0 do
		(
			if selfAtt[j].controller != undefined then append SelfAtt_Param  selfAtt[j].controller
				else
			(
			messagebox ("this Modifier Has \"Undefined\" Custom Attribute Parameters") title:"Undefined Self Attribute"
				SelfAtt_Param =#()
				test = 1
			)
		)
	)
	
)

fn save_vals typeOf =
(
	
	case TypeOf of
	(
		"pos":
		(
				
			state_num = this.AttHolderRollout.ddl_states.selection
			if Position_param.count != 0 do
			(
				valsNum = Position_param.count
				Pos_Indexnum[state_num] = valsNum
				
				getIndexes "pos"
				
				cnt = 1
				for i =  indexStart to indexEnd do
				(
					Position_param_vals[i] = Position_param[cnt].value
					cnt +=1
				)
				
			)
		)
		"rot":
		(
			
			state_num = this.AttHolderRollout.ddl_states.selection
			if rotation_param.count != 0 do
			(
				valsNum = rotation_param.count
				Rot_Indexnum[state_num] = valsNum
				
				getIndexes "rot"
				
				cnt = 1
				for i =  indexStart to indexEnd do
				(
					rotation_param_vals[i] = rotation_param[cnt].value as matrix3
					cnt +=1
				)
				
			)
		)
		"CustParam":
		(
			--messagebox ""
				
			state_num = this.AttHolderRollout.ddl_states.selection
			if float_Param.count != 0 do
			(
				valsNum = float_Param.count
				float_Indexnum[state_num] = valsNum
				
				getIndexes "CustParam"
				
				cnt = 1
				for i =  indexStart to indexEnd do
				(
					float_Param_val[i] = float_Param[cnt].value
					cnt +=1
				)
				
			)
		)
		"SelfCustAtt":
		(
			state_num = this.AttHolderRollout.ddl_states.selection
			if SelfAtt_Param.count != 0 do
			(
				valsNum = SelfAtt_Param.count
				selfAtt_Indexnum[state_num] = valsNum
				
				getIndexes "SelfCustAtt"
				
				cnt = 1
				for i =  indexStart to indexEnd do
				(
					SelfAtt_Param_val[i] = SelfAtt_Param[cnt].value
					cnt +=1
				)
			)
		)
	)

)

fn load_vals TypeOf =
(
	state_num = this.AttHolderRollout.ddl_states.selection
	
	case typeOf of
	(
		
		"pos":
		(
			if Position_param_vals.count != 0 do
			(
				/*
				valsNum = Position_param.count
				Pos_Indexnum[state_num] = valsNum
				*/
				getIndexes "pos"
				
				cnt = 1
				for i =  indexStart to indexEnd do
				(
					 Position_param[cnt].value = Position_param_vals[i] 
					cnt +=1
				)
				
			)
		)
		"rot":
		(
			
			if Rotation_param_vals.count != 0 do
			(
				/*
				valsNum = Position_param.count
				Pos_Indexnum[state_num] = valsNum
				*/
				getIndexes "rot"
				
				cnt = 1
				for i =  indexStart to indexEnd do
				(
					 Rotation_param[cnt].value = Rotation_param_vals[i] as eulerAngles
					cnt +=1
					
				)
				
			)
		)
		"CustParam":
		(
			if float_Param_val.count != 0 do
			(
				/*
				valsNum = Position_param.count
				Pos_Indexnum[state_num] = valsNum
				*/
				getIndexes "CustParam"
				
				cnt = 1
				for i =  indexStart to indexEnd do
				(
					 float_Param[cnt].value = float_Param_val[i] 
					cnt +=1
				)
				
			)
		)
		"SelfCustAtt":
		(
			if SelfAtt_Param_val.count != 0 do
			(
				/*
				valsNum = Position_param.count
				Pos_Indexnum[state_num] = valsNum
				*/
				getIndexes "SelfCustAtt"
				
				cnt = 1
				for i =  indexStart to indexEnd do
				(
					 SelfAtt_Param[cnt].value = SelfAtt_Param_val[i] 
					cnt +=1
				)
				
			)
		)
	)
)

fn Delete_state_val =
(
	state_num = this.AttHolderRollout.ddl_states.selection
	if Position_param_vals.count != 0 do
	(	
		getIndexes "pos"
		for i =indexStart to indexEnd do
		(
			deleteItem Position_param_vals indexStart
		)
		deleteItem Pos_Indexnum state_num
	)
	
	if Rotation_param_vals.count != 0 do
	(
		getIndexes "rot"
		for i =indexStart to indexEnd do
		(
			deleteItem Rotation_param_vals indexStart
		)
		deleteItem Rot_Indexnum state_num
	)
	
	if float_Param_val.count != 0 do
	(
		getIndexes "CustParam"
		for i =indexStart to indexEnd do
		(
			deleteItem float_Param_val indexStart
		)
		deleteItem Float_Indexnum state_num
	)
	
	if SelfAtt_Param_val.count != 0 do
	(
		getIndexes "SelfCustAtt"
		for i =indexStart to indexEnd do
		(
			deleteItem SelfAtt_Param_val indexStart
		)
		deleteItem selfAtt_Indexnum state_num
	)
)

fn Double_ctrl_check type input_controller=
(
	local false_state = 0
	case type of 
	(
		pos:
		(
		for x in Position_param while false_state == 0 where x==input_controller do 
			false_state = 1
		)
		rotation:
		(
		for x in rotation_param while false_state == 0 where x==input_controller do 
			false_state = 1
		)
		float:
		(
			for x in float_Param while false_state == 0 where  x==input_controller do
				#(false_state = 1, print "false")
				--if x==input_controller do false_state = 1
		)
		
	
	)
	if false_state == 0 then return true else return false
)

fn slidermerge_initiate typeOf =

(
	state_num = this.AttHolderRollout.ddl_states.selection
	case typeOf of
	(
		"pos":
		(
			finalPosVals=#()
			initialPosVals=#()
			getIndexes "pos"
			if Position_param.count != 0 do
			(
				
				
				cnt = 1
				for i =  indexStart to indexEnd do
				(
					 finalPosVals[cnt] = Position_param_vals[i] 
					cnt +=1
				)
				for i =1 to (indexEnd-indexStart)+1 do
				(
					initialPosVals[i] = Position_param[i].value
				)
				
			)
			
		)
		"rot":
		(
				finalRotVals=#()
				initialRotVals=#()
			getIndexes "rot"
			if Rotation_param.count != 0 do
			(
				cnt = 1
				for i =  indexStart to indexEnd do
				(
					
					 finalRotVals[cnt] = Rotation_param_vals[i] as eulerAngles
					cnt +=1
				)
				for i =1 to (indexEnd-indexStart)+1 do
				(
					
					initialRotVals[i] = Rotation_param[i].value as eulerangles
				)
			)
		)
		"CustAtt":
		(
			initialAttVals=#()
			finalAttVals=#()
			getIndexes "CustParam"
			
			if float_Param.count !=0 do
			(
				cnt = 1
				for i = indexStart to indexEnd do
				(
					finalAttVals[cnt] = float_Param_val[i]
					cnt+=1
				)
				for i =1 to (indexEnd-indexStart)+1 do
				(
					initialAttVals[i] = float_Param[i].value
				)
			)
			
		)
		"SelfAtt":
			(
				initialSelfAttVals = #()
				finalSelfAttVals = #()
				getIndexes "SelfCustAtt"
				if SelfAtt_Param.count !=0 do
				(
					cnt = 1
					for i = indexStart to indexEnd do
					(
						finalSelfAttVals[cnt] = SelfAtt_Param_val[i]
						cnt+=1
					)
					for i =1 to (indexEnd-indexStart)+1 do
					(
						initialSelfAttVals[i] = SelfAtt_Param[i].value
					)
				)
			)
	)
)
fn slidermerge_merge typeOf val=
(
	case typeOf of
	(
		"pos":
		(
			getIndexes "pos"
			
			if Position_param.count != 0 do
			(
				for i= 1 to ((indexEnd-indexStart)+1) do
				(
				--val = this.AttHolderRollout.sld_merge.value
				diff =  finalPosVals[i] - initialPosVals[i] 
				div = diff / 100.0
				mergeVal = div *val 
				Position_param[i].value = (initialPosVals[i] + mergeVal )
				)
			)
		)
		"rot":
		(
			getIndexes "rot"
			if rotation_param.count != 0 do
			(
				for i= 1 to ((indexEnd-indexStart)+1) do
				(
					Diff_x = finalRotvals[i].x - initialRotvals[i].x
					Diff_y = finalRotvals[i].y - initialRotvals[i].y
					Diff_z = finalRotvals[i].z - initialRotvals[i].z
					Div_x = Diff_x/100
					Div_y = Diff_y/100
					Div_z = Diff_z/100
					mergeVal_x = Div_x * val
					mergeVal_y = Div_y * val
					mergeVal_z = Div_z * val
					ini_x = initialRotvals[i].x
					ini_y = initialRotvals[i].y
					ini_z = initialRotvals[i].z
					--mergeVal = stringstream ""
					Rotation_param[i].value = (eulerAngles (ini_x+mergeVal_x) (ini_y+mergeVal_y) (ini_z+mergeVal_z) )
				)
				
			)
		)
		"CustAtt":
		(
			getIndexes "CustParam"
			if float_Param.count!=0 do
			(
				for i= 1 to ((indexEnd-indexStart)+1) do
				(
					diff =  finalAttVals[i] - initialAttVals[i] 
					div = diff / 100.0
					mergeVal = div *val 
					float_Param[i].value = (initialAttVals[i] + mergeVal )
				)
			)
		)
		"SelfCustAtt":
		(
			getIndexes "SelfCustAtt"
			if SelfAtt_Param.count!=0 do
			(
				for i= 1 to ((indexEnd-indexStart)+1) do
				(
					diff =  finalSelfAttVals[i] - initialSelfAttVals[i] 
					div = diff / 100.0
					mergeVal = div *val 
					SelfAtt_Param[i].value = (initialSelfAttVals[i] + mergeVal )
				)
			)
		)
	)
)
parameters UIparam rollout:AttHolderRollout
(
	checks	type:#booltab tabsizevariable:false	tabsize:7 ui:(chk_pos, chk_rotation,chk_custAtt ,chk_SelfAtt,chk_Lpos,chk_Lrot,chk_Lcust) default:true
	ddl_items	type:#stringtab	tabsizevariable:true  tabsize:0
	subChecks type:#booltab tabsizevariable:false 	tabsize:2 ui:(chk_getZeroPos,chk_getZeroRot)
	subEnabled	type:#booltab tabsizevariable:false 	tabsize:2 default:true
)

fn get_list = 
	(
		for i = 1 to this.ddl_states.items.count do 
			append ddl_items this.ddl_states.items[i]
	)
	fn set_list =
	(
		a= #()
		for i = 1 to ddl_items.count do
			append a ddl_items[i]
		
			 this.AttHolderRollout.ddl_states.items = a
		if this.AttHolderRollout.ddl_states.items.count > 0 do
			this.AttHolderRollout.ddl_states.selection = this.AttHolderRollout.ddl_states.items.count
	)
	rollout rolloutabout "About" width:203 height:199
	(
		label lbl1 "Armin Attribute Holder" pos:[13,15] width:150 height:120
		button btn_visit "Visit Website" pos:[51,135] width:94 height:22
		
		on rolloutabout open do
		(
			
			lbl1.text = "\"Armin Attribute Holder\" V.1.0\n\n"
			lbl1.text += "Created by: Armin Soleiman Boroujerdi\n\n"
			lbl1.text +="www.ArminSoleimani.net\n\n"
			lbl1.text += "Armin.Soleimani.87@gmail.com"
		)
		on btn_visit pressed do
		(
			shelllaunch "http://www.arminsoleimani.net/start/cg-animation/plug-ins/" ""
		)
	)
	rollout AttHolderRollout "Armin Attribute Holder" width:162 height:280
	(
	checkbox chk_pos "Get Position" pos:[31,23] width:97 height:17
	checkbox chk_rotation "Get Rotation" pos:[30,64] width:97 height:17
	checkbox chk_custAtt "Get Custom Attributes" pos:[30,104] width:131 height:17
	checkbox chk_SelfAtt "Include Self Attributes" pos:[30,124] width:130 height:17
	dropdownList ddl_states "" pos:[40,380] width:118 height:21
	button btn_save "Store" pos:[32,228] width:73 height:34
	edittext edt_name "Name:" pos:[40,358] width:118 height:17
	button btn_delete "Delete" pos:[32,267] width:73 height:34
	button btn_load "Restore" pos:[112,228] width:73 height:34
	slider sld_merge "" pos:[37,431] width:150 height:25 ticks:10
	GroupBox grp1 "Define" pos:[6,3] width:200 height:188
	GroupBox grp2 "Presets" pos:[8,201] width:200 height:328
	label lbl5 "Merge :" pos:[46,414] width:110 height:14
	button btn_Reset "Reset" pos:[112,267] width:73 height:34
	checkbox chk_Lpos "Position" pos:[50,466] width:56 height:19
	checkbox chk_Lrot "Rotation" pos:[110,465] width:64 height:19
	checkbox chk_Lcust "Cust Atts" pos:[50,486] width:73 height:16
	button btn_get "GET" pos:[49,145] width:116 height:34
	checkbox chk_getZeroPos "Get Frozen Pos" pos:[54,41] width:100 height:16
	checkbox chk_getZeroRot "Get Frozen Rotation" pos:[54,82] width:120 height:16
	button btn_select "Select Preset Objects" pos:[32,308] width:151 height:20
	button btn_key "Key Current Preset" pos:[32,331] width:151 height:20
		
		on btn_key pressed do
		(
			if chk_Lpos.state ==true do KeyPreset "pos"
			if chk_Lrot.state ==true do KeyPreset "rot"	
			if chk_Lcust.state ==true do keypreset "CustParam"
		)
		on btn_select pressed do
		(
			deselect objects
			/*
			deselect objects
			for x in objects_col do
				selectmore x
			*/
			if chk_Lpos.state ==true do ReturnObjects "pos"
			if chk_Lrot.state ==true do ReturnObjects "rot"
			if chk_Lcust.state ==true do
				for x in CustAttobjects_col do selectmore x
				--print CustAttobjects_col
		)
			
		on chk_pos changed state do
		(
			chk_getZeroPos.enabled = state
		)
		on chk_rotation changed state do
		(
			chk_getZeroRot.enabled = state
		)
		
		on sld_merge buttondown do
		(
			if chk_Lpos.state == true and  Position_param.count != 0 do slidermerge_initiate "pos"
			if chk_Lrot.state == true and  rotation_param.count != 0 do slidermerge_initiate "rot"
			if chk_Lcust.state == true do  
			(
				if float_Param.count !=0 do slidermerge_initiate "CustAtt"
				if SelfAtt_Param.count!=0 do  slidermerge_initiate "SelfAtt"
			)
			
		)
		on sld_merge changed val do
		(
			if chk_Lpos.state == true and  Position_param.count != 0 do slidermerge_merge "pos" val
			if chk_Lrot.state == true and  rotation_param.count != 0 do slidermerge_merge "rot" val
			 if chk_Lcust.state == true do
			 (
				if  float_Param.count !=0  do slidermerge_merge "CustAtt" val
				if  SelfAtt_Param.count !=0  do slidermerge_merge "SelfCustAtt" val	
			 )				
			
		)
			on sld_merge buttonup do
		(
			sld_merge.value = 0
			
		)
		on btn_load pressed do
		(
			load_vals "pos"
			load_vals "rot"
			load_vals "CustParam"
			load_vals "SelfCustAtt"
		)
		
		on btn_save pressed do
		(
			if this.AttHolderRollout.edt_name.text != "" do 
			(
			append ddl_items (this.AttHolderRollout.edt_name.text)
			set_list()
			save_vals "pos"
			save_vals "rot"
			save_vals "CustParam"
			save_vals	"SelfCustAtt"
				this.AttHolderRollout.edt_name.text = ""
			)
		)
		
		on btn_delete pressed do
		(
			Delete_state_val()
			
			if ddl_items.count >0 do
				deleteItem ddl_items this.AttHolderRollout.ddl_states.selection
			set_list()
		)
		
		on AttHolderRollout open do
		(
			chk_getZeroPos.enabled = subEnabled[1]
			chk_getZeroRot.enabled = subEnabled[2]
			set_list()
			thisObject = $
		)
		
		on AttHolderRollout close do
		(
			subEnabled[1] = chk_getZeroPos.enabled 
			subEnabled[2] = chk_getZeroRot.enabled
		)
		on btn_get pressed do
		(
			
			testSelf = 0 
			--thisObj = undefined 
			for x in selection do 
				if x ==thisObject do #(testSelf = 1)
			if testSelf== 1 then
				messageBox ("Cannot include " + thisObject.name+ " , Please Deselect it." ) title:"Selfie Error"
			else
			(
				posParamNum = 0
				rotParamNum = 0
				attParamNum = 0
				--selfattParamNum = 0
				posParamNum = Position_param.count
				rotParamNum = rotation_param.count
				attParamNum = float_Param.count
				--selfattParamNum = SelfAtt_Param.count
			if chk_pos.state == true do get_Pos_Param()
			if chk_rotation.state == true do get_Rotation_param()
			if chk_custAtt.state == true do get_custAtt()
			if chk_SelfAtt.state== true do  Get_currentAtt()
			
				ss  = stringstream ""
				format "Number of Positions Added:\n" to:ss
				format "%"  ((Position_param.count- posParamNum) as string +"\n\n" )  to:ss 
				format "Number of Rotations Added:\n" to:ss
				format "%"  ((rotation_param.count- rotParamNum) as string +"\n\n" )  to:ss 
				format "Number of Floats Added:\n" to:ss
				format "%"  ((float_Param.count- attParamNum) as string +"\n\n" )  to:ss 
				format "Number of current Attributes :\n" to:ss
				format "%"  ((SelfAtt_Param.count) as string )  to:ss 
				
				messagebox ss beep:false title:"Attributes info"
			)
		)
			
		on btn_Reset pressed do
		(
			a = querybox "This will eliminate all the presets and defined \ncontrollers and is NOT UNDO-ABLE.\nProceed?" beep:true
			
			if a == true then
			(
				Position_param=#()
				Position_param_vals=#()
				Pos_Indexnum=#()
				
				rotation_param=#()
				rotation_param_vals=#()
				Rot_Indexnum=#()
				
				float_Param=#()
				float_Param_val=#()
				float_Indexnum=#()
				
				SelfAtt_Param=#()
				SelfAtt_Param_val =#()
				selfAtt_Indexnum =#()
				
				ddl_items=#()
				
				set_list()
			)
			else
			messagebox "Phew! that was close..." beep:false title:"live another day"
		)
		
	)
	on create do 
	(

	)
)
